package com.taichuan.hardwaredemo;

import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;

import com.taichuan.hardwaredemo.entity.ReadDeviceIdEntity;
import com.taichuan.hardwaredemo.service.L10HardwareService;
import com.taichuan.hardwaredemo.service.L13HardwareService;
import com.taichuan.serialcommunication.SerialPortPublic;
import com.taichuan.serialcommunication.utils.SystemUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 太川硬件SDKDemo 主Activity
 * 适用于以下设备：
 * A33 L10设备
 * A64 L10Pro设备
 */
public class L13MainActivity extends AppCompatActivity {
    private static final String TAG = L13MainActivity.class.getSimpleName();
    private Intent intent;

    public static final int SERIAL_INIT_RESULT = 1;      //太川串口库初始化结果
    public static final int SERIAL_DOOR_STATE = 2;       //门检开关状态
    public static final int SERIAL_TAMPER_STATE = 3;     //防拆报警状态
    public static final int SERIAL_KEY_CODE = 4;         //物理按键事件
    public static final int SERIAL_READ_HARD_NUM = 5;    //设备机身号读取结果
    public static final int SERIAL_READ_CARD = 6;        //刷卡模块刷卡结果
    public static final int SERIAL_HARD_VERSION = 7;     //设备单片机程序版本
    public static final int SERIAL_MICROWAVE_STATE = 8;  //微波模块触发状态

    private static TextView serialInitTv;
    private static ImageView doorStateIv;
    private static ImageView tamperStateIv;
    private static TextView keyCodeTv;
    private static TextView readHardNumTv;
    private static TextView readCardTv;
    private static TextView serialVersionTv;
    private static ImageView microWaveStateIv;
    private static TextView wifiInfoTv;
    private Button openOrCloseFillLightBtn;
    private Button openOrCloseBuzzerBtn;
    private EditText hardwareNumEt;
    private LinearLayout cardMcuLayout;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main_for_l13);
        initViews();

        //
        /**
         * 初始化硬件功能Service（建议使用Service来管理硬件功能的监听回调，也可自行使用其他方式实现）
         */
        intent = new Intent(this, L13HardwareService.class);

        startService(intent);
    }

    private void initViews(){
        serialInitTv = findViewById(R.id.serial_init_state_textView);
        doorStateIv = findViewById(R.id.serial_door_state_imageView);
        tamperStateIv = findViewById(R.id.serial_tamper_state_imageView);
        keyCodeTv = findViewById(R.id.serial_key_code_textView);
        readHardNumTv = findViewById(R.id.serial_read_hard_num_textView);
        readCardTv = findViewById(R.id.serial_read_card_textView);
        serialVersionTv = findViewById(R.id.serial_hard_version_textView);
        microWaveStateIv = findViewById(R.id.serial_microwave_state_imageView);

        openOrCloseFillLightBtn = findViewById(R.id.open_or_close_fill_light);
        openOrCloseFillLightBtn.setTag("open");
        openOrCloseBuzzerBtn = findViewById(R.id.open_or_close_buzzer);
        openOrCloseBuzzerBtn.setTag("open");

        hardwareNumEt = findViewById(R.id.hardware_num_input_editText);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        //注销硬件功能Service
        stopService(intent);
    }


    public void onXmlClilck(View view) {
        switch (view.getId()){
            case R.id.unlock_button:{       //开锁
                SerialPortPublic.unlock();
                Toast.makeText(this, "开锁时耳朵靠近设备，能听到继电器开关的声音", Toast.LENGTH_SHORT).show();
                break;
            }
            case R.id.open_or_close_fill_light:{        //开关补光灯
                if (openOrCloseFillLightBtn.getTag().equals("open")){
                    //开灯
                    SerialPortPublic.controlFillLight(true);
                    openOrCloseFillLightBtn.setTag("close");
                    openOrCloseFillLightBtn.setText("关补光灯");
                    Toast.makeText(this, "开灯", Toast.LENGTH_SHORT).show();
                }else {
                    //关灯
                    SerialPortPublic.controlFillLight(false);
                    openOrCloseFillLightBtn.setTag("open");
                    openOrCloseFillLightBtn.setText("开补光灯");
                    Toast.makeText(this, "关灯", Toast.LENGTH_SHORT).show();
                }
                break;
            }
            case R.id.open_or_close_buzzer:{            //蜂鸣器控制（需循环响蜂鸣器清自行实现）
                if (openOrCloseBuzzerBtn.getTag().equals("open")){
                    //短音蜂鸣器
                    SerialPortPublic.controlBuzzer(1);
                    openOrCloseBuzzerBtn.setTag("close");
                    openOrCloseBuzzerBtn.setText("长音蜂鸣器");
                    Toast.makeText(this, "短音蜂鸣器", Toast.LENGTH_SHORT).show();
                }else {
                    //长音蜂鸣器
                    SerialPortPublic.controlBuzzer(0);
                    openOrCloseBuzzerBtn.setTag("open");
                    openOrCloseBuzzerBtn.setText("短音蜂鸣器");
                    Toast.makeText(this, "长音蜂鸣器", Toast.LENGTH_SHORT).show();
                }
                break;
            }

            /**
             * 写入机身号至单片机中存储，机身号生成规则请自行设计
             * 写入的机身号必须为14位时设备单片机程序才能正确根据传入的机身号来生成以太网Mac地址
             * 【此处只是将生成的以太网Mac地址存入单片机中，如何设置以太网Mac地址到以太网网卡请自行查询相关文章】
             * 写入的机身号可以为纯数字，也可以为 字母+13位数字 | 2位字母+12位数字 的组合，其余组合无法将机身号写入到单片机
             * Eg. A1234567894567 \ AC123456789456 | 91234567894562
             */
            case R.id.write_hardware_num_button:{
                String hardwareNum = hardwareNumEt.getText().toString().trim();
                boolean isSucess = SerialPortPublic.writeHardwareNum(hardwareNum);
                Toast.makeText(this, "写入机身号执行结果：  " + isSucess, Toast.LENGTH_SHORT).show();
                break;
            }

            case R.id.reboot_device_button:{                        //重启设备功能
                Toast.makeText(this, "重启设备", Toast.LENGTH_SHORT).show();
                SerialPortPublic.reboot();
                break;
            }

            case R.id.check_door_state_button:{                     //主动查询门检开关状态，查询后将在回调中接收
                SerialPortPublic.checkDoorState();
                break;
            }

            case R.id.check_tamper_state_button:{                     //主动查询防拆开关状态，查询后将在回调中接收
                SerialPortPublic.checkTamperState();
                break;
            }
        }
    }

    /**
     * 定义一个Handler用于处理硬件功能Service回调出的硬件消息事件
     * 此处仅为Demo演示，消息事件的传递方案可自行决定
     */
    public static Handler mainHandler = new Handler(new Handler.Callback() {
        @Override
        public boolean handleMessage(@NonNull Message msg) {
            switch (msg.what){
                case SERIAL_INIT_RESULT:{               //太川串口库初始化结果
                    boolean isInitSucess = (boolean) msg.obj;
                    serialInitTv.setText(isInitSucess ? "成功" : "失败");
                    break;
                }
                case SERIAL_DOOR_STATE:{                //门检开关状态
                    boolean isOpen = (boolean) msg.obj;
                    doorStateIv.setSelected(isOpen);
                    break;
                }
                case SERIAL_TAMPER_STATE:{              //防拆报警状态
                    boolean isTamper = (boolean) msg.obj;
                    tamperStateIv.setSelected(isTamper);
                    break;
                }
                case SERIAL_KEY_CODE:{                  //物理按键事件
                    int keycode = (int) msg.obj;
                    keyCodeTv.setText(keycode + "");
                    break;
                }
                case SERIAL_READ_HARD_NUM:{             //设备机身号读取结果
                    ReadDeviceIdEntity entity = (ReadDeviceIdEntity) msg.obj;
                    readHardNumTv.setText(entity.getDeviceNum() + "  以太网网卡Mac： " + entity.getEthernetMac());
                    break;
                }
                case SERIAL_READ_CARD:{                 //刷卡模块刷卡结果
                    int outType = msg.arg1;
                    boolean isOut = false;
                    if (outType == 1){
                        isOut = true;
                    }else {
                        isOut = false;
                    }
                    String cardNum = (String) msg.obj;
                    //根据机型处理卡号数据
                    if (Build.VERSION.SDK_INT == Build.VERSION_CODES.KITKAT) {          //A33 L10 设备
                        cardNum = cardNum.substring(0, 8);
                        readCardTv.setText(cardNum + " 是否外置刷卡器： " + (isOut ? "是" : "否"));
                    }else if (Build.VERSION.SDK_INT == Build.VERSION_CODES.M) {    //A64 L10Pro 设备
                        //获取被刷的卡号是否 能在离线刷卡功能存储区中找到该卡号
                        boolean isOfflineMatch = cardNum.length() == 10 && cardNum.endsWith("01");
                        cardNum = cardNum.substring(0, 8);
                        readCardTv.setText(cardNum + " 是否外置刷卡器： " + (isOut ? "是" : "否") + " 能否离线刷卡功能中找到该卡号： " + isOfflineMatch);
                    }else {
                        readCardTv.setText(cardNum + " 是否外置刷卡器： " + (isOut ? "是" : "否"));
                    }
                    break;
                }
                case SERIAL_HARD_VERSION:{              //设备单片机程序版本
                    String hardVerison = (String) msg.obj;
                    serialVersionTv.setText(hardVerison);
                    break;
                }
                case SERIAL_MICROWAVE_STATE:{           //微波模块触发状态（L10，L10 Pro 设备没有微波模块，无需理会）
                    boolean havePeople = (boolean) msg.obj;
                    microWaveStateIv.setSelected(havePeople);
                    break;
                }
            }
            return false;
        }
    });
}
