package com.taichuan.hardwaredemo;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;

import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.g3288.g3288board.TcRk3399GpioClient;
import com.taichuan.hardwaredemo.entity.ReadDeviceIdEntity;
import com.taichuan.hardwaredemo.service.TC5000DXL4HardwareService;
import com.taichuan.hardwaredemo.util.HexUtils;
import com.taichuan.serialcommunication.SerialPortPublic;

public class TC5000DXL4MainActivity extends AppCompatActivity {
    private static final String TAG = TC5000DXL4MainActivity.class.getSimpleName();
    private Intent intent;

    public static final int SERIAL_INIT_RESULT = 1;      //太川串口库初始化结果
    public static final int SERIAL_DOOR_STATE = 2;       //门检开关状态
    public static final int SERIAL_READ_HARD_NUM = 4;    //设备机身号读取结果
    public static final int SERIAL_HARD_VERSION = 5;     //设备单片机程序版本
    public static final int SERIAL_READ_485_DATA = 6;   //485数据
    public static final int SERIAL_TEMP_DATA = 7;       //测温模块温度数据
    public static final int SERIAL_READ_CARD = 8;        //刷卡模块刷卡结果
    public static final int SERIAL_OUT_SWITCH = 9;        //外出开关状态
    public static final int SERIAL_MICROWAVE_STATE = 10;  //微波模块触发状态


    private static TextView serialInitTv;
    private static ImageView doorStateIv;
    private static ImageView outStateIv;
    private static ImageView microWaveStateIv;
    private static TextView readHardNumTv;
    private static TextView serialVersionTv;
    private static TextView readCardTv;
    private Button openOrCloseFillLightBtn;
    private Button openOrCloseInfraredLightBtn;
    private Button openOrCloseBuzzerBtn;
    private EditText hardwareNumEt;
    private static TextView cardFor485Tv;
    private static TextView tempTv;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_maint_for_tc_5000d_xl4);
        initViews();

        /**
         * 初始化硬件功能Service（建议使用Service来管理硬件功能的监听回调，也可自行使用其他方式实现）
         * TC-5000D-XL4 设备 设备请参考 Rk3399Temp
         */
        Log.d(TAG, "onCreate: 1111");
        intent = new Intent(this, TC5000DXL4HardwareService.class);

        startService(intent);
    }

    private void initViews(){
        serialInitTv = findViewById(R.id.serial_init_state_textView);
        doorStateIv = findViewById(R.id.serial_door_state_imageView);
        outStateIv = findViewById(R.id.serial_out_state_imageView);
        microWaveStateIv = findViewById(R.id.serial_microwave_state_imageView);
        readHardNumTv = findViewById(R.id.serial_read_hard_num_textView);
        serialVersionTv = findViewById(R.id.serial_hard_version_textView);
        readCardTv = findViewById(R.id.serial_read_card_textView);
        tempTv = findViewById(R.id.temp_data_textView);

        openOrCloseFillLightBtn = findViewById(R.id.open_or_close_fill_light);
        openOrCloseFillLightBtn.setTag("open");
        openOrCloseInfraredLightBtn = findViewById(R.id.open_or_close_infrared_light);
        openOrCloseInfraredLightBtn.setTag("open");
        openOrCloseBuzzerBtn = findViewById(R.id.open_or_close_buzzer);
        openOrCloseBuzzerBtn.setTag("open");

        hardwareNumEt = findViewById(R.id.hardware_num_input_editText);

        cardFor485Tv = findViewById(R.id.serial_read_485_card_textView);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        //注销硬件功能Service
        stopService(intent);
    }

    public void onXmlClilck(View view) {
        switch (view.getId()){
            case R.id.unlock_button:{       //开锁
                SerialPortPublic.unlock();
                Toast.makeText(this, "开锁时耳朵靠近设备，能听到继电器开关的声音", Toast.LENGTH_SHORT).show();
                break;
            }
            case R.id.open_or_close_fill_light:{        //开关补光灯
                if (openOrCloseFillLightBtn.getTag().equals("open")){
                    //开灯
                    TcRk3399GpioClient.getInstance().openOrCloseFillLight(true);
                    openOrCloseFillLightBtn.setTag("close");
                    openOrCloseFillLightBtn.setText("关补光灯");
                    Toast.makeText(this, "开灯", Toast.LENGTH_SHORT).show();
                }else {
                    //关灯
                    TcRk3399GpioClient.getInstance().openOrCloseFillLight(false);
                    openOrCloseFillLightBtn.setTag("open");
                    openOrCloseFillLightBtn.setText("开补光灯");
                    Toast.makeText(this, "关灯", Toast.LENGTH_SHORT).show();
                }
                break;
            }
            case R.id.open_or_close_infrared_light:{        //开关红外灯
                if (openOrCloseInfraredLightBtn.getTag().equals("open")){
                    //开红外灯
                    TcRk3399GpioClient.getInstance().openOrCloseInfraredLight(true);
                    openOrCloseInfraredLightBtn.setTag("close");
                    openOrCloseInfraredLightBtn.setText("关红外灯");
                    Toast.makeText(this, "开红外灯", Toast.LENGTH_SHORT).show();
                }else {
                    //关红外灯
                    TcRk3399GpioClient.getInstance().openOrCloseInfraredLight(false);
                    openOrCloseInfraredLightBtn.setTag("open");
                    openOrCloseInfraredLightBtn.setText("开红外灯");
                    Toast.makeText(this, "关红外灯", Toast.LENGTH_SHORT).show();
                }
                break;
            }
            case R.id.open_or_close_buzzer:{            //蜂鸣器控制（需循环响蜂鸣器清自行实现）
                if (openOrCloseBuzzerBtn.getTag().equals("open")){
                    //短音蜂鸣器
                    SerialPortPublic.controlBuzzer(1);
                    openOrCloseBuzzerBtn.setTag("close");
                    openOrCloseBuzzerBtn.setText("长音蜂鸣器");
                    Toast.makeText(this, "短音蜂鸣器", Toast.LENGTH_SHORT).show();
                }else {
                    //长音蜂鸣器
                    SerialPortPublic.controlBuzzer(0);
                    openOrCloseBuzzerBtn.setTag("open");
                    openOrCloseBuzzerBtn.setText("短音蜂鸣器");
                    Toast.makeText(this, "长音蜂鸣器", Toast.LENGTH_SHORT).show();
                }
                break;
            }

            /**
             * 写入机身号至单片机中存储，机身号生成规则请自行设计
             * 写入的机身号必须为14位时设备单片机程序才能正确根据传入的机身号来生成以太网Mac地址
             * 【此处只是将生成的以太网Mac地址存入单片机中，如何设置以太网Mac地址到以太网网卡请自行查询相关文章】
             * 写入的机身号可以为纯数字，也可以为 字母+13位数字 | 2位字母+12位数字 的组合，其余组合无法将机身号写入到单片机
             * Eg. A1234567894567 \ AC123456789456 | 91234567894562
             */
            case R.id.write_hardware_num_button:{
                String hardwareNum = hardwareNumEt.getText().toString().trim();
                boolean isSucess = SerialPortPublic.writeHardwareNum(hardwareNum);
                Toast.makeText(this, "写入机身号执行结果：  " + isSucess, Toast.LENGTH_SHORT).show();
                break;
            }

            case R.id.reboot_device_button:{                        //重启设备功能
                Toast.makeText(this, "重启设备", Toast.LENGTH_SHORT).show();
                SerialPortPublic.reboot();
                break;
            }

        }
    }


    /**
     * 定义一个Handler用于处理硬件功能Service回调出的硬件消息事件
     * 此处仅为Demo演示，消息事件的传递方案可自行决定
     */
    public static Handler mainHandler = new Handler(new Handler.Callback() {
        @Override
        public boolean handleMessage(@NonNull Message msg) {
            switch (msg.what){
                case SERIAL_INIT_RESULT:{               //太川串口库初始化结果
                    boolean isInitSucess = (boolean) msg.obj;
                    serialInitTv.setText(isInitSucess ? "成功" : "失败");
                    break;
                }
                case SERIAL_DOOR_STATE:{                //门检开关状态
                    boolean isOpen = (boolean) msg.obj;
                    doorStateIv.setSelected(isOpen);
                    break;
                }
                case SERIAL_OUT_SWITCH:{                //外出开关状态
                    boolean isOpen = (boolean) msg.obj;
                    outStateIv.setSelected(isOpen);
                    break;
                }
                case SERIAL_MICROWAVE_STATE:{
                    boolean havePeople = (boolean) msg.obj;
                    microWaveStateIv.setSelected(havePeople);
                    break;
                }

                case SERIAL_READ_HARD_NUM:{             //设备机身号读取结果
                    ReadDeviceIdEntity entity = (ReadDeviceIdEntity) msg.obj;
                    readHardNumTv.setText(entity.getDeviceNum() + "  以太网网卡Mac： " + entity.getEthernetMac());
                    break;
                }
                case SERIAL_READ_485_DATA:{
                    String data = (String) msg.obj;
                    //如果读取到的485卡号数据小于16位，需要将数据补全至16位，才能与刷卡功能刷出来的数据一致
                    String cardNum = HexUtils.getStandardCard(data);
                    cardFor485Tv.setText(cardNum);
                    break;
                }
                case SERIAL_HARD_VERSION:{              //设备单片机程序版本
                    String hardVerison = (String) msg.obj;
                    serialVersionTv.setText(hardVerison);
                    break;
                }
                case SERIAL_TEMP_DATA:{                 //测温模块温度数据
                    String tempValue = (String) msg.obj;
                    tempTv.setText(tempValue);
                    break;
                }
                case SERIAL_READ_CARD:{                 //刷卡模块刷卡结果
                    int outType = msg.arg1;
                    boolean isOut = false;
                    if (outType == 1){
                        isOut = true;
                    }else {
                        isOut = false;
                    }
                    String cardNum = (String) msg.obj;
                    readCardTv.setText(cardNum + " 是否外置刷卡器： " + (isOut ? "是" : "否"));
                    break;
                }
            }
            return false;
        }
    });

}