package com.taichuan.hardwaredemo.service;

import android.app.Service;
import android.content.Intent;
import android.os.Build;
import android.os.IBinder;
import android.util.Log;
import com.a33.a33board.A33GpioForWifiPropeActionDefine;
import com.a33.a33board.TcA33GpioForWifiPropeClient;
import com.a64.a64board.A64GpioActionDefine;
import com.a64.a64board.TcA64GpioClient;
import com.taichuan.hardwaredemo.A33A64H1MainActivity;
import com.taichuan.hardwaredemo.entity.ReadDeviceIdEntity;
import com.taichuan.serialcommunication.ISerialPort2DataListener;
import com.taichuan.serialcommunication.ISerialPortActionListener;
import com.taichuan.serialcommunication.ISerialPortMcuCardListener;
import com.taichuan.serialcommunication.SDKInitListener;
import com.taichuan.serialcommunication.SerialPortPublic;
import java.nio.charset.StandardCharsets;

/**
 * A33 L10设备/A64 L10 Pro设备专用 硬件功能Service
 */
public class L10HardwareService extends Service {
    private static final String TAG = L10HardwareService.class.getSimpleName();

    @Override
    public void onCreate() {
        super.onCreate();
        Log.e(TAG, "onCreate: 正在执行onCreate()---");
        //延时3秒再启动太川串口库，防止与硬件云Launcher冲突
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    Thread.sleep(3000);
                    //初始化太川串口库
                    initSerialPort();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }).start();

        //初始化太川gpio库
        initGpio();
    }

    @Override
    public IBinder onBind(Intent intent) {
        Log.e(TAG, "onBind: onBind()---");
        return null;
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        Log.e(TAG, "onStartCommand: onStartCommand()---");
        /**
         * Demo演示用，因此不允许自动重启Service
         * 实际接入项目使用时建议允许自动重启
         */
        return START_NOT_STICKY;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        //注销串口库事件
        destorySerialPort();
        //注销太川串口库
        SerialPortPublic.uninit(this);
        //注销太川gpio库
        destoryGpio();
    }

    /**
     * 定义一个方法用于初始化串口库
     */
    protected void initSerialPort() {
        //添加太川串口库SDK 初始化结果监听器
        SerialPortPublic.setSdkInitListener(mSdkInitListener);
        //添加太川串口库SDK 硬件消息事件监听器
        SerialPortPublic.addSerialPortActionListener(mSerialPortActionListener);
        //添加太川串口库SDK Wifi探针数据接收监听器
        SerialPortPublic.addSerialPortForWifiPropeDataListener(mSerialPort2ForWifiPropeListener);
        //添加太川串口库SDK 单片机离线刷卡功能执行结果监听器
        SerialPortPublic.addSerialPortMcuCardListener(mSerialPortMcuCardListener);

        //初始化太川串口库SDK
        /**
         * @param context 上下文对象
         * @param debug 是否关闭看门狗，true为关闭看门狗，false为开启看门狗
         * @param ignoreCheckSum 是否对单片机上报的协议数据进行 校验和 校验，true为开启校验，false为不进行校验和校验
         * @param isNeedCheckIoState 是否需要在串口库SDK初始化时 进行防拆、门检等IO口的状态检查
         * @param isInit485         是否初始化485串口
         * @param isInitSerialport2 是否初始化串口2
         * @param isInitSerialportForWifiPrope 是否初始化 WiFi探针串口
         */
        SerialPortPublic.initSerialPortMulti(this, true, true,true, false,
                false,true);
    }

    protected void destorySerialPort(){
        //移除太川串口库SDK 初始化结果监听器
        SerialPortPublic.setSdkInitListener(null);
        //移除太川串口库SDK 硬件消息事件监听器
        SerialPortPublic.removeSerialPortActionListener(mSerialPortActionListener);
        //移除太川串口库SDK Wifi探针数据接收监听器
        SerialPortPublic.removeSerialPortForWifiPropeDataListener(mSerialPort2ForWifiPropeListener);
        //移除太川串口库SDK 单片机离线刷卡功能执行结果监听器
        SerialPortPublic.removeSerialPortMcuCardListener(mSerialPortMcuCardListener);
    }

    /**
     * 定义一个方法用于根据设备类型型号决定是否需要初始化gpio库
     */
    private void initGpio() {
        //根据设备类型型号决定是否需要初始化Gpio库
        if (Build.VERSION.SDK_INT == Build.VERSION_CODES.KITKAT) {          //A33 L10 设备
            //A33的gpio库无需初始化，直接单例调用方法即可
            //开启Wifi探针需要复位 Wifi探针的gpio口
            int result = TcA33GpioForWifiPropeClient.getInstance().LedControl(A33GpioForWifiPropeActionDefine.ACTION_WIFI_PROBE_RESET);
            Log.e(TAG, "initGpio:  A33 L10设备 Wifi探针 重置结果：  " + result);

        } else if (Build.VERSION.SDK_INT == Build.VERSION_CODES.M) {    //A64 L10Pro 设备
            //初始化A64 L10Pro 设备 Wifi探针 太川gpio库
            // 第一个参数为是否开启gpio读取线程，第二个参数为是否开启debug模式
            int result = TcA64GpioClient.getInstance().initGpio(false, false);
            Log.e(TAG, "initGpio: 初始化 A64 L10Pro 设备 Wifi探针 太川gpio库结果：  " + result);

            //开启Wifi探针需要复位 Wifi探针的gpio口
            int resultReset = TcA64GpioClient.getInstance().controlGpio(A64GpioActionDefine.ACTION_WIFI_PROBE_RESET_A64);
            int resultEnable = TcA64GpioClient.getInstance().controlGpio(A64GpioActionDefine.ACTION_WIFI_PROBE_ENABLE_A64);
            Log.e(TAG, "initGpio: c A64 L10Pro 设备 Wifi探针 重置结果：  " + resultReset + "  开启结果：  " + resultEnable);
        } else {                                                                         //未知设备

        }
    }

    /**
     * 定义一个方法用于根据设备类型型号决定是否需要注销gpio库
     */
    private void destoryGpio() {
        //根据设备类型型号决定是否需要初始化Gpio库
        if (Build.VERSION.SDK_INT == Build.VERSION_CODES.KITKAT) {          //A33 L10 设备
            //A33的gpio库无需注销

        } else if (Build.VERSION.SDK_INT == Build.VERSION_CODES.M) {    //A64 L10Pro 设备
            //注销事件监听
            TcA64GpioClient.getInstance().removeFaceInitedListener(TAG);
            //注销gpio库
            int result = TcA64GpioClient.getInstance().destoryGpio();
            Log.e(TAG, "destoryGpio: 注销 A64 L10Pro 设备 Wifi探针 太川gpio库 结果：  " + result);
        } else {                                                                         //未知设备

        }
    }

    /**
     * 太川串口库SDK 初始化结果监听器
     */
    private final SDKInitListener mSdkInitListener = new SDKInitListener() {
        @Override
        public void SDKInited() {
            Log.e(TAG, "SDKInited: 太川串口库SDK 初始化成功");
            A33A64H1MainActivity.mainHandler.sendMessage(A33A64H1MainActivity.mainHandler.obtainMessage
                    (A33A64H1MainActivity.SERIAL_INIT_RESULT, true));

            //太川串口库SDK 初始化成功，读取机身号
            SerialPortPublic.readHardwareNum();
        }

        @Override
        public void SDKLosted() {
            Log.e(TAG, "SDKLosted: 太川串口库SDK 初始化失败");
            A33A64H1MainActivity.mainHandler.sendMessage(A33A64H1MainActivity.mainHandler.obtainMessage
                    (A33A64H1MainActivity.SERIAL_INIT_RESULT, false));
        }
    };

    /**
     * 太川串口库SDK 硬件消息事件监听器
     */
    private final ISerialPortActionListener mSerialPortActionListener = new ISerialPortActionListener() {
        /**
         * 门检开关状态
         * @param isOn true 门状态为开，false 门状态关
         */
        @Override
        public void onDoorSwitchStatus(boolean isOn) {
            Log.e(TAG, "onDoorSwitchStatus: 门检开关状态：" + (isOn ? "开" : "关"));
            A33A64H1MainActivity.mainHandler.sendMessage(A33A64H1MainActivity.mainHandler.obtainMessage
                    (A33A64H1MainActivity.SERIAL_DOOR_STATE, isOn));
        }

        @Override
        public void onDoorOpenStatus(boolean b) {
            Log.e(TAG, "onDoorOpenStatus: 门开关状态：" + (b ? "开" : "关"));
        }

        /**
         * 防拆报警状态
         * @param isTamper true 设备被拆除，报警；false 设备正常
         */
        @Override
        public void onTamperAlways(boolean isTamper) {
            Log.e(TAG, "onTamperAlways: 防拆状态：" + isTamper);
            A33A64H1MainActivity.mainHandler.sendMessage(A33A64H1MainActivity.mainHandler.obtainMessage
                    (A33A64H1MainActivity.SERIAL_TAMPER_STATE, isTamper));
        }

        @Override
        public void onOutSwitch(boolean b) {

        }

        /**
         * 物理按键事件
         * @param i 按下的键值
         */
        @Override
        public void onPhysicsKeyDown(int i) {
            Log.e(TAG, "onPhysicsKeyDown: 键盘按下的键值：" + i);
            A33A64H1MainActivity.mainHandler.sendMessage(A33A64H1MainActivity.mainHandler.obtainMessage
                    (A33A64H1MainActivity.SERIAL_KEY_CODE, i));
        }

        @Override
        public void onLightPowerResult(int i, int i1) {

        }

        @Override
        public void onWriteDeviceIdResult(boolean b) {
            Log.e(TAG, "onWriteDeviceIdResult: 机身号写入结果：  " + b);
            if (b){
                //延时1秒重新读取机身号
                new Thread(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            Thread.sleep(1000);
                            SerialPortPublic.readHardwareNum();
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    }
                }).start();

            }
        }

        /**
         * 设备机身号读取结果（需自行写入机身号）
         * @param b 是否读取成功
         * @param s 设备机身号
         * @param s1 设备以太网Mac地址
         */
        @Override
        public void onReadDeviceIdResult(boolean b, String s, String s1) {
            Log.e(TAG, "onReadDeviceIdResult: 机身号读取结果：" + b + "，设备号：" + s + "，以太网MAC：" + s1);
            A33A64H1MainActivity.mainHandler.sendMessage(A33A64H1MainActivity.mainHandler.obtainMessage
                    (A33A64H1MainActivity.SERIAL_READ_HARD_NUM, new ReadDeviceIdEntity(b, s, s1)));

            //机身号读取成功，开始读取单片机版本号
            SerialPortPublic.getSpiVersion();
        }

        /**
         * 刷卡模块刷卡事件
         * @param b 是否为外置刷卡器
         * @param s 刷卡卡号
         */
        @Override
        public void onCardResult(boolean b, String s) {
            Log.e(TAG, "onCardResult: 刷卡卡号：" + s + "，是否为外置刷卡器：" + b);
            A33A64H1MainActivity.mainHandler.sendMessage(A33A64H1MainActivity.mainHandler.obtainMessage
                    (A33A64H1MainActivity.SERIAL_READ_CARD, b ? 1 : 0, -1, s));
        }

        @Override
        public void onBlueToothResult(String s) {

        }

        /**
         * 设备单片机程序版本
         * @param s 设备单片机程序版本
         */
        @Override
        public void onVersion(String s) {
            Log.e(TAG, "onVersion: 设备单片机程序版本：" + s);
            A33A64H1MainActivity.mainHandler.sendMessage(A33A64H1MainActivity.mainHandler.obtainMessage
                    (A33A64H1MainActivity.SERIAL_HARD_VERSION, s));
        }

        /**
         * 微波模块触发状态
         * @param b true为有物体经过，触发微波模块
         */
        @Override
        public void onACCheck(boolean b) {
            Log.e(TAG, "onACCheck: 微波模块触发状态：" + b);
            A33A64H1MainActivity.mainHandler.sendMessage(A33A64H1MainActivity.mainHandler.obtainMessage
                    (A33A64H1MainActivity.SERIAL_MICROWAVE_STATE, b));
        }

        @Override
        public void onLowPowerCheck(boolean b) {

        }

        @Override
        public void onBodyTemp(int i, int i1) {

        }

        @Override
        public void onMicrowaveResult(boolean b) {

        }
    };

    /**
     * 太川串口库SDK 单片机离线刷卡功能执行结果事件监听器
     * 只有L10 Pro设备才有效
     */
    private final ISerialPortMcuCardListener mSerialPortMcuCardListener = new ISerialPortMcuCardListener() {
        @Override
        public void onAddCardResult(boolean isAddSucess) {
            Log.e(TAG, "onAddCardResult: 添加卡号到单片机结果：  " + isAddSucess);
        }

        @Override
        public void onDeleteAllCardResult(boolean isDeleteSucess) {
            Log.e(TAG, "onAddCardResult: 从单片机删除所有卡号结果：  " + isDeleteSucess);
        }

        @Override
        public void onDeleteCardResult(boolean isDeleteSucess) {
            Log.e(TAG, "onAddCardResult: 从单片机删除单张卡号结果：  " + isDeleteSucess);
        }
    };

    /**
     * 太川串口库SDK WiFi探针搜索事件监听器
     */
    private final ISerialPort2DataListener mSerialPort2ForWifiPropeListener =
            new ISerialPort2DataListener() {

                @Override
                public void receivedData(byte[] bytes) {
                    String result = new String(bytes, StandardCharsets.UTF_8);
//                    Log.e(TAG, "receivedData: 接收到Wifi探针数据：  " + result);
                    A33A64H1MainActivity.mainHandler.sendMessage(A33A64H1MainActivity.mainHandler.obtainMessage
                            (A33A64H1MainActivity.SERIAL_WIFI_PROPE_INFO, result));
                }
            };
}
