package com.taichuan.hardwaredemo.service;

import android.app.Service;
import android.content.Intent;
import android.os.IBinder;
import android.util.Log;

import com.g3288.g3288board.TcRk3399GpioClient;
import com.taichuan.hardwaredemo.Rk3399TempMainActivity;
import com.taichuan.hardwaredemo.entity.ReadDeviceIdEntity;
import com.taichuan.serialcommunication.ISerialPortActionListener;
import com.taichuan.serialcommunication.SDKInitListener;
import com.taichuan.serialcommunication.SerialPortPublic;

/**
 * Rk3399 TC-5000D-B1 测温门禁设备 专用 硬件功能Service
 */
public class Rk3399TempHardwareService extends Service {
    private static final String TAG = Rk3399TempHardwareService.class.getSimpleName();

    @Override
    public void onCreate() {
        super.onCreate();
        Log.e(TAG, "onCreate: 正在执行onCreate()---");
        //延时3秒再启动太川串口库，防止与硬件云Launcher冲突,如设备出厂没有自带太川硬件云Launcher程序可不执行延迟
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    Thread.sleep(3000);
                    //初始化太川串口库
                    initSerialPort();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }).start();

        //初始化太川gpio库
        initGpio();
    }

    @Override
    public IBinder onBind(Intent intent) {
        Log.e(TAG, "onBind: onBind()---");
        return null;
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        Log.e(TAG, "onStartCommand: onStartCommand()---");
        /**
         * Demo演示用，因此不允许自动重启Service
         * 实际接入项目使用时建议允许自动重启
         */
        return START_NOT_STICKY;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        //注销串口库事件
        destorySerialPort();
        //注销太川Gpio库
        destoryGpio();
        //注销太川串口库
        SerialPortPublic.uninit(this);
    }

    /**
     * 定义一个方法用于初始化串口库
     */
    protected void initSerialPort() {
        //添加太川串口库SDK 初始化结果监听器
        SerialPortPublic.setSdkInitListener(mSdkInitListener);
        //添加太川串口库SDK 硬件消息事件监听器
        SerialPortPublic.addSerialPortActionListener(mSerialPortActionListener);

        //初始化太川串口库SDK
        /**
         * @param context 上下文对象
         * @param debug 是否关闭看门狗，true为关闭看门狗，false为开启看门狗
         *              设备送测及产品上线阶段强烈建议开启看门狗功能，提高设备运行稳定性
         *              当设备出现异常、死机、系统卡死、门禁App异常崩溃时会在30秒左右自动重启设备，保障设备恢复正常
         * @param ignoreCheckSum 是否对单片机上报的协议数据进行 校验和 校验，true为开启校验，false为不进行校验和校验
         * @param isNeedCheckIoState 是否需要在串口库SDK初始化时 进行防拆、门检等IO口的状态检查
         */
        SerialPortPublic.initWith485(this,false,true,true);
    }

    protected void destorySerialPort(){
        //移除太川串口库SDK 初始化结果监听器
        SerialPortPublic.setSdkInitListener(null);
        //移除太川串口库SDK 硬件消息事件监听器
        SerialPortPublic.removeSerialPortActionListener(mSerialPortActionListener);
    }

    /**
     * 定义一个方法用于初始化 太川gpio库
     */
    private void initGpio() {
        int result = TcRk3399GpioClient.getInstance().initGpio(false);

        Log.e(TAG, "initGpio: Rk3399 设备 初始化gpio库结果： " + result);
    }

    /**
     * 定义一个方法用于注销 太川gpio库
     */
    private void destoryGpio() {
        int result = TcRk3399GpioClient.getInstance().destoryGpio();
        Log.e(TAG, "destoryGpio: 注销 Rk3399 gpio库结果：  " + result);
    }

    /**
     * 太川串口库SDK 初始化结果监听器
     */
    private final SDKInitListener mSdkInitListener = new SDKInitListener() {
        @Override
        public void SDKInited() {
            Log.e(TAG, "SDKInited: 太川串口库SDK 初始化成功");
            Rk3399TempMainActivity.mainHandler.sendMessage(Rk3399TempMainActivity.mainHandler.obtainMessage
                    (Rk3399TempMainActivity.SERIAL_INIT_RESULT, true));

            //太川串口库SDK 初始化成功，读取机身号
            SerialPortPublic.readHardwareNum();
        }

        @Override
        public void SDKLosted() {
            Log.e(TAG, "SDKLosted: 太川串口库SDK 初始化失败");
            Rk3399TempMainActivity.mainHandler.sendMessage(Rk3399TempMainActivity.mainHandler.obtainMessage
                    (Rk3399TempMainActivity.SERIAL_INIT_RESULT, false));
        }
    };

    /**
     * 太川串口库SDK 硬件消息事件监听器
     */
    private final ISerialPortActionListener mSerialPortActionListener = new ISerialPortActionListener() {
        /**
         * 门检开关状态
         * @param isOn true 门状态为开，false 门状态关
         */
        @Override
        public void onDoorSwitchStatus(boolean isOn) {
        }

        @Override
        public void onDoorOpenStatus(boolean b) {
            Log.e(TAG, "onDoorOpenStatus: 门开关状态：" + (b ? "开" : "关"));
        }

        /**
         * 防拆报警状态
         * @param isTamper true 设备被拆除，报警；false 设备正常
         */
        @Override
        public void onTamperAlways(boolean isTamper) {
        }

        @Override
        public void onOutSwitch(boolean b) {

        }

        /**
         * 物理按键事件
         * @param i 按下的键值
         */
        @Override
        public void onPhysicsKeyDown(int i) {
        }

        @Override
        public void onLightPowerResult(int i, int i1) {

        }

        @Override
        public void onWriteDeviceIdResult(boolean b) {
            Log.e(TAG, "onWriteDeviceIdResult: 机身号写入结果：  " + b);
            if (b){
                //延时1秒重新读取机身号
                new Thread(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            Thread.sleep(1000);
                            SerialPortPublic.readHardwareNum();
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    }
                }).start();

            }
        }

        /**
         * 设备机身号读取结果（需自行写入机身号）
         * @param b 是否读取成功
         * @param s 设备机身号
         * @param s1 设备以太网Mac地址
         */
        @Override
        public void onReadDeviceIdResult(boolean b, String s, String s1) {
            Log.e(TAG, "onReadDeviceIdResult: 机身号读取结果：" + b + "，设备号：" + s + "，以太网MAC：" + s1);
            Rk3399TempMainActivity.mainHandler.sendMessage(Rk3399TempMainActivity.mainHandler.obtainMessage
                    (Rk3399TempMainActivity.SERIAL_READ_HARD_NUM, new ReadDeviceIdEntity(b, s, s1)));

            //机身号读取成功，开始读取单片机版本号
            SerialPortPublic.getSpiVersion();
        }

        /**
         * 刷卡模块刷卡事件
         * @param b 是否为外置刷卡器
         * @param s 刷卡卡号
         */
        @Override
        public void onCardResult(boolean b, String s) {
            Log.e(TAG, "onCardResult: 刷卡卡号：" + s + "，是否为外置刷卡器：" + b);
            Rk3399TempMainActivity.mainHandler.sendMessage(Rk3399TempMainActivity.mainHandler.obtainMessage
                    (Rk3399TempMainActivity.SERIAL_READ_CARD, b ? 1 : 0, -1, s));
        }

        @Override
        public void onBlueToothResult(String s) {

        }

        /**
         * 设备单片机程序版本
         * @param s 设备单片机程序版本
         */
        @Override
        public void onVersion(String s) {
            Log.e(TAG, "onVersion: 设备单片机程序版本：" + s);
            Rk3399TempMainActivity.mainHandler.sendMessage(Rk3399TempMainActivity.mainHandler.obtainMessage
                    (Rk3399TempMainActivity.SERIAL_HARD_VERSION, s));
        }

        /**
         * 微波模块触发状态
         * @param b true为有物体经过，触发微波模块
         */
        @Override
        public void onACCheck(boolean b) {
            Log.e(TAG, "onACCheck: 微波模块触发状态：" + b);
        }

        @Override
        public void onLowPowerCheck(boolean b) {

        }

        @Override
        public void onBodyTemp(int temp, int distance) {
            String it = (Integer) (temp / 10) + "";
            String de = (Integer) (temp % 10) + "";
            String result = String.format("测温模块温度数据： %s.%s", it, de);
            Log.i(TAG, result);
            Rk3399TempMainActivity.mainHandler.sendMessage(Rk3399TempMainActivity.mainHandler.obtainMessage
                    (Rk3399TempMainActivity.SERIAL_TEMP_DATA, it + "." + de));
        }

        @Override
        public void onMicrowaveResult(boolean b) {

        }
    };
}
