package com.taichuan.hardware.cloud.demo;

import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;

import com.taichuan.mqtt.EquipmentPublic;
import com.taichuan.mqtt.IConnectionListener;
import com.taichuan.mqtt.IEquipmentFunctionListener;
import com.taichuan.mqtt.VKCode;
import com.taichuan.mqtt.VKService;
import com.taichuan.mqtt.a.e;
import com.taichuan.serialcommunication.ISerialPortActionListener;
import com.taichuan.serialcommunication.SDKInitListener;
import com.taichuan.serialcommunication.SerialPortPublic;

public class MainActivity extends AppCompatActivity {
    private final static String TAG = MainActivity.class.getSimpleName();
    private TextView connectStatusTV;
    private TextView deviceNumberTV;
    private TextView swipeCardTV;
    private TextView serialPortStatusTV;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        initViews();
    }

    private void initViews() {

        connectStatusTV = findViewById(R.id.tv_connect_status);
        deviceNumberTV = findViewById(R.id.tv_device_number);
        swipeCardTV = findViewById(R.id.tv_swipe_card);
        serialPortStatusTV = findViewById(R.id.tv_serial_port_status);

        //初始化硬件串口SDK
        initSerialPort();

        //初始化硬件云SDK，连接硬件云
        initHardwareCloud();
    }

    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.bt_unlock:
                SerialPortPublic.unlock();
                break;
            case R.id.bt_reboot:
                SerialPortPublic.reboot();
                break;
            case R.id.bt_short_buzzer:
                SerialPortPublic.controlBuzzer(1);
                break;
            case R.id.bt_long_buzzer:
                SerialPortPublic.controlBuzzer(0);
                break;
            default:
                break;
        }

    }

    /**
     * 初始化硬件串口SDK，即可控制设备硬件
     */
    private void initSerialPort() {
        Log.i(TAG, "onCreate: 串口通讯SDK版本：" + SerialPortPublic.getsdkInfo());
        //添加太川串口库SDK 初始化结果监听器
        SerialPortPublic.setSdkInitListener(mSdkInitListener);
        //添加太川串口库SDK 硬件消息事件监听器
        SerialPortPublic.addSerialPortActionListener(mSerialPortActionListener);

        //初始化太川串口库SDK
        /**
         * @param context  上下文对象
         * @param b  是否关闭看门狗，true为关闭看门狗，false为开启看门狗
         * @param b1 是否对单片机上报的协议数据进行 校验和 校验，true为开启校验，false为不进行校验和校验强烈建议设置为true，如果发现单片机上报数据校验和校验失败的，应敦促单片机程序修改，而不是设置为false忽略
         * @param b2 是否需要在串口库SDK初始化时 进行防拆、门检等IO口的状态检查
         * @param b3 是否初始化485串口
         * @param b4 是否初始化串口2
         * @param b5 是否初始化 WiFi探针串口
         */
        SerialPortPublic.initSerialPortMulti(this, true, true, true,
                false, false, false);

    }

    /**
     * 初始化硬件云,建立与硬件云平台MQTT连接
     */
    private void initHardwareCloud() {
        VKService.addConnectionListener(iConnectionListener);
        EquipmentPublic.addEquipmentFunctionListener(iEquipmentFunctionListener);
        //建议开启与硬件云平台校时，接收从MQTT平台指令
        EquipmentPublic.init(getApplicationContext(), true);
    }

    /**
     * 太川硬件串口SDK 初始化结果监听器
     */
    private final SDKInitListener mSdkInitListener = new SDKInitListener() {
        @Override
        public void SDKInited() {
            Log.e(TAG, "SDKInited: 太川串口库SDK 初始化成功");
            runOnUiThread(() -> {
                serialPortStatusTV.setText("成功");
                serialPortStatusTV.setTextColor(getResources().getColor(R.color.green));
            });
            SerialPortPublic.readHardwareNum();
        }

        @Override
        public void SDKLosted() {
            Log.e(TAG, "SDKLosted: 太川串口库SDK 初始化失败");
            runOnUiThread(() -> {
                serialPortStatusTV.setText("失败");
                serialPortStatusTV.setTextColor(getResources().getColor(R.color.red));
            });
        }
    };

    /**
     * 太川硬件串口SDK 硬件消息事件监听器
     */
    private final ISerialPortActionListener mSerialPortActionListener = new ISerialPortActionListener() {
        /**
         * 门检开关状态
         * @param isOn true 门状态为开，false 门状态关
         */
        @Override
        public void onDoorSwitchStatus(boolean isOn) {
            Log.e(TAG, "onDoorSwitchStatus: 门检开关状态：" + (isOn ? "开" : "关"));
        }

        @Override
        public void onDoorOpenStatus(boolean b) {
            Log.e(TAG, "onDoorOpenStatus: 门开关状态：" + b);
        }

        /**
         * 防拆报警状态
         * @param isTamper true 设备被拆除，报警；false 设备正常
         */
        @Override
        public void onTamperAlways(boolean isTamper) {
            Log.e(TAG, "onTamperAlways: 防拆状态：" + isTamper);
        }

        @Override
        public void onOutSwitch(boolean b) {

        }

        /**
         * 物理按键事件(针对按键款门口机)
         * @param i 按下的键值
         */
        @Override
        public void onPhysicsKeyDown(int i) {
            Log.e(TAG, "onPhysicsKeyDown: 键盘按下的键值：" + i);

        }

        @Override
        public void onLightPowerResult(int i, int i1) {

        }

        @Override
        public void onWriteDeviceIdResult(boolean b) {
            Log.e(TAG, "onWriteDeviceIdResult: 机身号写入结果：  " + b);
        }

        /**
         * 设备机身号读取结果
         * @param b 是否读取成功
         * @param s 设备机身号
         * @param s1 设备以太网Mac地址
         */
        @Override
        public void onReadDeviceIdResult(boolean b, String s, String s1) {
            Log.e(TAG, "onReadDeviceIdResult: 机身号读取结果：" + b + "，设备号：" + s + "，以太网MAC：" + s1);
            runOnUiThread(() -> {
                deviceNumberTV.setText(String.format(getString(R.string.device_number), s));
                if (deviceNumberTV.getVisibility() == View.GONE) {
                    deviceNumberTV.setVisibility(View.VISIBLE);
                }
            });
        }

        /**
         * 刷卡模块刷卡事件
         * @param b 是否为外置刷卡器
         * @param s 刷卡卡号
         */
        @Override
        public void onCardResult(boolean b, String s) {
            Log.e(TAG, "onCardResult: 刷卡卡号：" + s + "，是否为外置刷卡器：" + b);
            runOnUiThread(() -> {
                swipeCardTV.setText(String.format(getString(R.string.swipe_card), s, b ? "是" : "否"));
                if (swipeCardTV.getVisibility() == View.GONE) {
                    swipeCardTV.setVisibility(View.VISIBLE);
                }
            });
        }

        @Override
        public void onBlueToothResult(String s) {

        }

        /**
         * 设备单片机程序版本
         * @param s 设备单片机程序版本
         */
        @Override
        public void onVersion(String s) {
            Log.e(TAG, "onVersion: 设备单片机程序版本：" + s);

        }

        /**
         * 微波模块触发状态
         * @param b true为有物体经过，触发微波模块
         */
        @Override
        public void onACCheck(boolean b) {
            Log.e(TAG, "onACCheck: 微波模块触发状态：" + b);

        }

        @Override
        public void onLowPowerCheck(boolean b) {

        }

        @Override
        public void onBodyTemp(int i, int i1) {

        }

        @Override
        public void onMicrowaveResult(boolean b) {

        }
    };

    /**
     * MQTT连接状态监听
     */
    IConnectionListener iConnectionListener = new IConnectionListener() {
        @Override
        public void onConnectInitialized(VKCode vkCode, String... strings) {
        }


        @Override
        public void onConnected() {
            runOnUiThread(() -> {
                connectStatusTV.setText("已连接");
                connectStatusTV.setTextColor(getResources().getColor(R.color.green));
            });

        }

        @Override
        public void onConnectedFailure(String s) {
            runOnUiThread(() -> {
                connectStatusTV.setText("连接失败");
                connectStatusTV.setTextColor(getResources().getColor(R.color.red));
            });

        }

        @Override
        public void onConnectComplete(boolean b) {
            runOnUiThread(() -> {
                connectStatusTV.setText("已连接");
                connectStatusTV.setTextColor(getResources().getColor(R.color.green));
            });
        }

        @Override
        public void onConnectionLost() {
            runOnUiThread(() -> {
                connectStatusTV.setText("失去连接");
                connectStatusTV.setTextColor(getResources().getColor(R.color.red));
            });
        }
    };

    /**
     * 接收硬件云平台发送过来的自定义MQTT消息/指令
     */
    IEquipmentFunctionListener iEquipmentFunctionListener = new IEquipmentFunctionListener() {
        @Override
        public void onEquipmentCustomData(long l, String s) {
            Log.i(TAG, "IEquipmentFunctionListener→onEquipmentCustomData;L = " + l + ",s = " + s);

        }

        @Override
        public boolean onEquipmentUnlock(String s) {
            //收到开锁指令，执行开锁
            SerialPortPublic.unlock();
            //执行开锁后返回true
            return true;
        }

        @Override
        public void onEquipmentReboot() {

        }


        //MQTT消息处理超时回调
        @Override
        public void onMqttEventTimeout(String s, int i, int i1, int i2, long l, e e, Class aClass) {
            Log.i(TAG, "IEquipmentFunctionListener→onMqttEventTimeout;");

        }

    };
}